#include "exports.h"
#include "file.h"
#include "linkablemapobj.h"
#include "misc.h"
#include "texteditor.h"
#include "mainwindow.h"

extern Main *mainWindow;

ExportBase::ExportBase()
{
	indentPerDepth="  ";
}

void ExportBase::setPath (const QString &p)
{
	filepath=p;
}

void ExportBase::setMapCenter(MapCenterObj *mc)
{
	mapCenter=mc;
}

QString ExportBase::getSectionString(BranchObj *bostart)
{
	QString r;
	BranchObj *bo=bostart;
	int depth=bo->getDepth();
	while (depth>0)
	{
		r=QString("%1").arg(1+bo->getNum(),0,10)+"." + r;
		bo=(BranchObj*)(bo->getParObj());
		depth=bo->getDepth();
	}	
	if (r.isEmpty())
		return r;
	else	
		return r + " ";
}

void ExportBase::exportXML()
{
	QFile file (filepath);
	if ( !file.open( IO_WriteOnly ) )
	{
		// FIXME experimental, testing
		cout << "ExportBase::exportXML  couldn't open "<<filepath<<endl;
		return;
	}
	QTextStream ts( &file );	// use LANG decoding here...

	// Main loop over all branches
	QString s;
	QString actIndent("");
	int i;
	uint j;
	BranchObj *bo;
	bo=mapCenter->first();
	while (bo) 
	{
		// Make indentstring
		for (i=0;i<bo->getDepth();i++) actIndent+= indentPerDepth;

		// Write heading
		//	write (actIndent + getSectionString(bo) + bo->getHeading()+ "\n");
		if (bo->getDepth()==0)
		{
			ts << (bo->getHeading()+ "\n");
			for (j=0;j<bo->getHeading().length();j++) ts<<"=";
			ts << "\n";
		} else 	if (bo->getDepth()==1)
			ts << ("\n"+getSectionString(bo) + bo->getHeading()+ "\n");
		else	if (bo->getDepth()==2)
			ts << (actIndent + " o " + bo->getHeading()+ "\n");
		else	
			ts << (actIndent + " - " + bo->getHeading()+ "\n");
		
		// If necessary, write note
		if (!bo->getNote().isEmpty())
		{
			s =bo->getNoteASCII();
			s=s.replace ("\n","\n"+actIndent);
			ts << (s+"\n\n");
		}
		
		bo=bo->next();
		actIndent="";
	}
	file.close();
}

void ExportLaTeX::exportLaTeX() 
{
	// Exports a map to a LaTex file.  
	// This file needs to be included 
	// or inported into a LaTex document
	// it will not add a preamble, or anything 
	// that makes a full LaTex document.
  QFile file (filepath);
  if ( !file.open( IO_WriteOnly ) ) {
    // FIXME
    cout << "Export::exportMap  couldn't open "<<filepath<<endl;
    return;
  }
  QTextStream ts( &file );	// use LANG decoding here...
  ts.setEncoding (QTextStream::UnicodeUTF8); // Force UTF8
  
  // Main loop over all branches
  QString s;
  // QString actIndent("");
  // int i;
  BranchObj *bo;
  bo=mapCenter->first();
  while (bo) {
    if (bo->getDepth()==0);
    else if (bo->getDepth()==1) {
      ts << ("\\chapter{" + bo->getHeading()+ "}\n");
    }
    else if (bo->getDepth()==2) {
      ts << ("\\section{" + bo->getHeading()+ "}\n");
    }
    else if (bo->getDepth()==3) {
      ts << ("\\subsection{" + bo->getHeading()+ "}\n");
    }
    else if (bo->getDepth()==4) {
      ts << ("\\subsubsection{" + bo->getHeading()+ "}\n");
    }
    else {
      ts << ("\\paragraph*{" + bo->getHeading()+ "}\n");
    }
    
    // If necessary, write note
    if (!bo->getNote().isEmpty()) {
      ts << (bo->getNoteASCII());
      ts << ("\n");
    }
    
    bo=bo->next();
   }
  file.close();
}

ExportOO::ExportOO()
{
	// Create tmpdir and set paths
	tmpDir.setPath (makeUniqueDir("/tmp/vym-XXXXXX"));
	templateDir="exports/novell/template/";
	contentTemplateFile=templateDir+"content-template.xml";
	contentFile=tmpDir.path()+"/content.xml";
	pageTemplateFile=templateDir+"page-template.xml";
	chapterTemplateFile=templateDir+"chapter-template.xml";

	outputFile=tmpDir.currentDirPath()+"/test.odp";
}

ExportOO::~ExportOO()
{
	// Remove tmpdir
	removeDir (tmpDir);
}	

QString ExportOO::buildList (BranchObj *current)
{
    QString r;
    BranchObj *bo;

    uint i=0;
    bo=current->getFirstBranch();
    if (bo)
    {
        // Start list
        r+="<text:list text:style-name=\"L4\">\n";
        while (bo)
        {
            r+="<text:list-item><text:p >";
			r+=quotemeta(bo->getHeading());
			// If necessary, write note
			if (!bo->getNote().isEmpty())
				r+=bo->getNoteOpenDoc();
			r+="</text:p>";
			r+=buildList (bo);	// recursivly add deeper branches
            r+="</text:list-item>\n";
			i++;
			bo=current->getBranchNum(i);
        }
        r+="</text:list>\n";
    }
    return r;
}


void ExportOO::exportPresentation()
{
	QString allPages;

	bool useChapters=true;

	// Insert new content
	content.replace ("<!-- INSERT TITLE -->",quotemeta(mapCenter->getHeading()));
	content.replace ("<!-- INSERT AUTHOR -->",quotemeta(mapCenter->getAuthor()));

	QString	onePage;
	QString list;
	
	BranchObj *chapterBO=mapCenter->getFirstBranch();
    int i=0;
	BranchObj *pagesBO;
    int j=0;

	// Walk chapters
	while (chapterBO)
	{
		if (useChapters)
		{
			// Add page with chapter title
			onePage=chapterTemplate;
			onePage.replace ("<!-- INSERT PAGE HEADING -->", quotemeta(chapterBO->getHeading() ) );
			allPages+=onePage;
		} else
		{
			i=-2;	// only use inner loop to 
			        // turn mainbranches into pages
			chapterBO=mapCenter;
		}

		// Walk mainpages
		pagesBO=chapterBO->getFirstBranch();
		j=0;
		while (pagesBO)
		{
			// Add page with list of items
			onePage=pageTemplate;
			onePage.replace ("<!-- INSERT PAGE HEADING -->", quotemeta (pagesBO->getHeading() ) );
			list=buildList (pagesBO);
			onePage.replace ("<!-- INSERT LIST -->", list);
			allPages+=onePage;
			j++;
			pagesBO=chapterBO->getBranchNum(j);
		}
		i++;
		chapterBO=mapCenter->getBranchNum(i);
	}
	
	content.replace ("<!-- INSERT PAGES -->",allPages);

	// Write modified content
	QFile f (contentFile);
    if ( !f.open( IO_WriteOnly ) ) 
	{
		mainWindow->statusMessage(QString(QObject::tr("Could not write to %1")).arg(outputFile));
		return;
    }

    QTextStream t( &f );
    t << content;
    f.close();

	// zip tmpdir to destination
	zipDir (tmpDir,outputFile);	
}

void ExportOO::setConfigFile (const QString &cf)
{
	configFile=cf;
	int i=cf.findRev ("/");
	if (i>=0) configDir=cf.left(i);
	SimpleSettings set;
	set.readSettings(configFile);

	// set paths
	templateDir=configDir+"/"+set.readEntry ("Template");

	//FIXME check if templateDir is really set, otherwise abort

	contentTemplateFile=templateDir+"content-template.xml";
	contentFile=tmpDir.path()+"/content.xml";
	pageTemplateFile=templateDir+"page-template.xml";
	chapterTemplateFile=templateDir+"chapter-template.xml";

	outputFile=tmpDir.currentDirPath()+"/test.odp";
	
	// Copy template to tmpdir
	system ("cp -r "+templateDir+"* "+tmpDir.path());
	

	// Read content-template
	if (!loadStringFromDisk (contentTemplateFile,content))
	{
		qWarning ("Export::exportOO() Couldn't load from "+contentTemplateFile);
		return;
	}

	// Read page-template
	if (!loadStringFromDisk (pageTemplateFile,pageTemplate))
	{
		qWarning ("Export::exportOO() Couldn't load from "+pageTemplateFile);
		return;
	}
	
	// Read chapter-template
	if (!loadStringFromDisk (chapterTemplateFile,chapterTemplate))
	{
		qWarning ("Export::exportOO() Couldn't load from "+chapterTemplateFile);
		return;
	}

}

